/*
 *  ofxiPhoneImagePicker.cpp
 *  iPhone UIImagePicker Example
 *
 *  Created by Zach Gage on 3/1/09.
 *  Copyright 2009 stfj. All rights reserved.
 *
 */

#import "ofxiPhoneImagePicker.h"

//C++ class implementations

//--------------------------------------------------------------
ofxiPhoneImagePicker::ofxiPhoneImagePicker() {
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];		// MEMO
	imagePicker = [[ofxiPhoneImagePickerDelegate alloc] initWithPicker:this];
	cameraIsAvailable = [imagePicker isCameraAvailable];
	photoLibraryIsAvailable = [imagePicker isPhotoLibraryAvailable];
	savedPhotosIsAvailable = [imagePicker isSavedPhotosAvailable];
	pixels = NULL;
	maxDimension = 0;
	imageUpdated = false;
	imageCancelled = false;	// MEMO
	[pool release];		// MEMO
}

//--------------------------------------------------------------
void ofxiPhoneImagePicker::setMaxDimension(int _maxDimension)
{
	maxDimension = _maxDimension;
}

//--------------------------------------------------------------
ofxiPhoneImagePicker::~ofxiPhoneImagePicker()
{
	[imagePicker release];
	if(pixels) free(pixels);		// MEMO
}

//----------------------------------------------------------------
bool ofxiPhoneImagePicker::openCamera()
{
	return [imagePicker openCamera];
}
//----------------------------------------------------------------
bool ofxiPhoneImagePicker::openLibrary()
{
	return [imagePicker openLibrary];
}
//----------------------------------------------------------------
bool ofxiPhoneImagePicker::openSavedPhotos()
{
	return [imagePicker openSavedPhotos];
}
//----------------------------------------------------------------
int ofxiPhoneImagePicker::getOrientation()
{
	switch( [imagePicker getImageOrientation] )
	{
			case UIImageOrientationUp:
				return OFX_IMG_PICKER_UP;
				break;
			case UIImageOrientationDown:
				return OFX_IMG_PICKER_DOWN;
				break;
			case UIImageOrientationLeft:
				return OFX_IMG_PICKER_LEFT;
				break;
			case UIImageOrientationRight:
				return OFX_IMG_PICKER_RIGHT;
				break;
			default:
				return 0;
				break;

	}
}

//----------------------------------------------------------------
void ofxiPhoneImagePicker::loadPixels()
{	
	// load image texture
	CGImageRef photo;
	CGContextRef photoContext;
	GLubyte *photoData;
	
	photo = [imagePicker getCGImage];
	
	int bpp = 4;
	
	// Get the width and height of the image
	width = (int) CGImageGetWidth(photo);
	height = (int) CGImageGetHeight(photo);
	
	if(maxDimension != 0)
	{
		// Get the width and height of the image
		if(width > maxDimension || height > maxDimension)
		{
			if(width > height)
			{
				float change = (float)maxDimension/width;
				width = maxDimension;
				height = height * change;
			}
			else
			{
				float change = (float)maxDimension/height;
				height = maxDimension;
				width = width * change;
			}
		}
	}
	
	if(photo)
	{
		
		photoData = (GLubyte *) malloc(width * height * bpp);
		pixels = (unsigned char *) malloc(width * height * bpp);
		
		photoContext = CGBitmapContextCreate(photoData, width, height, 8, width * bpp, CGImageGetColorSpace(photo), kCGImageAlphaPremultipliedLast);
		
		
		CGContextDrawImage(photoContext, CGRectMake(0.0, 0.0, (CGFloat)width, (CGFloat)height), photo);
		
		// MEMO:
		int numBytesPerRow = width * bpp;
		for(int y=0; y<height; y++) {
			memcpy(pixels + (numBytesPerRow * y), photoData + (numBytesPerRow * (height - 1 - y)), numBytesPerRow);
		}
//
//		for(int x=0; x<width*height*bpp; x++)
//		{
//			pixels[x] = (unsigned char)photoData[x];
//		}
				
		CGContextRelease(photoContext);
		CGImageRelease(photo);
		free(photoData);
	}
	imageUpdated = true;
	
}
//----------------------------------------------------------------
void ofxiPhoneImagePicker::saveImage()
{
	[imagePicker saveImageToPhotoAlbum];
}
//----------------------------------------------------------------


// CLASS IMPLEMENTATIONS--------------objc------------------------
//----------------------------------------------------------------
@implementation ofxiPhoneImagePickerDelegate

- (id) initWithPicker:(ofxiPhoneImagePicker *) _picker
{
	if(self = [super init])
	{
		photoLibraryIsAvailable = false;
		savedPhotosIsAvailable = false;
		cameraIsAvailable = false;
		
		_imagePicker = [[UIImagePickerController alloc] init];
		
		_imagePicker.delegate = self;
		
		cppPixelLoader = _picker;
		
		
		if( [ UIImagePickerController isSourceTypeAvailable: UIImagePickerControllerSourceTypePhotoLibrary ]== YES )
		{
			photoLibraryIsAvailable = true;
		}
		
		if( [ UIImagePickerController isSourceTypeAvailable: UIImagePickerControllerSourceTypeCamera ]== YES )
		{
			cameraIsAvailable = true;
		}
		
		if( [ UIImagePickerController isSourceTypeAvailable: UIImagePickerControllerSourceTypeSavedPhotosAlbum ]== YES )
		{
			savedPhotosIsAvailable = true;
		}
		
		_image = [UIImage alloc];
	}
	return self;
}
//--------------------------------------------------------------

- (void)dealloc 
{ 
	[_imagePicker release];
	
//	[_image	release]; // MEMO: this isn't needed, gives EXC_BAD_ACCESS
	
	[super dealloc];
}

//----------------------------------------------------------------

- (void) imagePickerController:(UIImagePickerController *)picker didFinishPickingImage:(UIImage *)image editingInfo:(NSDictionary *)editingInfo
{	
	[_image initWithCGImage: image.CGImage];
	[_imagePicker.view removeFromSuperview];
	
	cppPixelLoader->loadPixels();
}

//--------------------------------------------------------------
- (void) imagePickerControllerDidCancel:(UIImagePickerController *)picker
{
	[_imagePicker.view removeFromSuperview];
	cppPixelLoader->imageCancelled = true;
}
//--------------------------------------------------------------
- (void) saveImageToPhotoAlbum
{
	UIImageWriteToSavedPhotosAlbum(_image, nil, nil, nil);
}
//--------------------------------------------------------------
- (bool) openCamera
{
	if(cameraIsAvailable)
	{
		[_imagePicker init]; //needs this to refresh the camera.
		_imagePicker.sourceType = UIImagePickerControllerSourceTypeCamera;
		[[[UIApplication sharedApplication] keyWindow] addSubview:_imagePicker.view];
		
		return true;
	}
	else
	{
		return false;
	}
}
//--------------------------------------------------------------
- (bool) openLibrary
{
	if(photoLibraryIsAvailable)
	{
		_imagePicker.sourceType = UIImagePickerControllerSourceTypePhotoLibrary;
		[[[UIApplication sharedApplication] keyWindow] addSubview:_imagePicker.view];
		
		return true;
	}
	else
	{
		return false;
	}
}
//--------------------------------------------------------------
- (bool) openSavedPhotos
{
	if(savedPhotosIsAvailable)
	{
		_imagePicker.sourceType = UIImagePickerControllerSourceTypeSavedPhotosAlbum;
		[[[UIApplication sharedApplication] keyWindow] addSubview:_imagePicker.view];
		
		return true;
	}
	else
	{
		return false;
	}
}

//--------------------------------------------------------------
- (UIImage *) getUIImage
{
	return _image;
}
//--------------------------------------------------------------
- (bool) isCameraAvailable
{
	return cameraIsAvailable;
}
//--------------------------------------------------------------
- (bool) isPhotoLibraryAvailable
{
	return photoLibraryIsAvailable;
}
//--------------------------------------------------------------
- (bool) isSavedPhotosAvailable
{
	return savedPhotosIsAvailable;
}

//--------------------------------------------------------------
- (CGImageRef) getCGImage;
{
	return _image.CGImage;
}
//--------------------------------------------------------------
- (int) getImageOrientation;
{
	return _image.imageOrientation;
}


@end